//camera function file
#include "main.h"
#define movement 0.03f

CVector3 Cross(CVector3 vVector1, CVector3 vVector2)
{
	CVector3 vNormal;//our soon to be normal vector

	// The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													// Get the X value
	vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y));
														
	// The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
	vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z));
														
	// The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
	vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x));

	// Return the cross product
	return vNormal;										 
}

float Magnitude(CVector3 vNormal)
{

	return (float)sqrt( (vNormal.x * vNormal.x) + 
						(vNormal.y * vNormal.y) + 
						(vNormal.z * vNormal.z) );
}


CVector3 Normalize(CVector3 vVector)
{
	// Get the magnitude of our normal
	float magnitude = Magnitude(vVector);				

	vVector = vVector / magnitude;
	
	// Finally, return our normalized vector
	return vVector;										
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		OUR CAMERA METHODS
//

//constructor
CCamera::CCamera()
{
	CVector3 vZero = CVector3(0.0, 0.0, 0.0);		// Init a vector to 0 0 0 for our position
	CVector3 vView = CVector3(0.0, 1.0, 0.5);		// Init a starting view vector (looking up and out the screen) 
	CVector3 vUp   = CVector3(0.0, 0.0, 1.0);		// Init a standard up vector (Rarely ever changes)
	
	//assigning are class varibles
	m_vPosition = vZero;
	m_vView	    = vView;
	m_vUp		= vUp;	 

}
//function  position normally called to initialise position
void CCamera::Position(float positionX, float positionY,  float positionZ,
					   float viewX,	    float viewY,	  float viewZ,
					   float upX,	    float upY,		  float upZ)
{
		CVector3 vPosition	= CVector3 (positionX, positionY, positionZ);
		CVector3 vView		= CVector3 (viewX, viewY, viewZ);
		CVector3 vUp		= CVector3 (upX, upY, upZ);

		m_vPosition = vPosition;	//sets the position 
		m_vView		= vView;	//sets the view
		m_vUp		= vUp;	//sets the up vector
}

//moving the camera ffrom the class
void CCamera::look()
{
	//updating camera movement
	gluLookAt(g_Camera.m_vPosition.x, g_Camera.m_vPosition.y, g_Camera.m_vPosition.z,	
			  g_Camera.m_vView.x,	  g_Camera.m_vView.y,	  g_Camera.m_vView.z,	
			  g_Camera.m_vUp.x,		  g_Camera.m_vUp.y,		  g_Camera.m_vUp.z);


}

//move the camera either forward or back
void CCamera::move (float speed)
{

	CVector3 VStorage =CVector3 (0.0, 0.0, 0.0); //initialize storage vector

	//create a vector from the origin take the head from the tale of the vector
	VStorage = m_vView - m_vPosition;

	//new position equals old position + (VStorage vector * speed)
	m_vPosition.x += VStorage.x * speed/2;
	m_vPosition.y += VStorage.y * speed/2;
	m_vPosition.z += VStorage.z * speed/2;

	m_vView.x	  += VStorage.x * speed/2;
	m_vView.y	  += VStorage.y * speed/2;
	m_vView.z	  += VStorage.z * speed/2;
}

//move the camera either up or down
void CCamera::moveUp (float speed)
{

	CVector3 VStorage =CVector3 (0.0, 0.0, 0.0); //initialize storage vector

	//create a vector from the origin take the head from the tale of the vector
	VStorage = m_vView - m_vPosition;

	//new position equals old position + (VStorage vector * speed)
	//m_vPosition.x += m_vUp.x * speed/2;
	m_vPosition.y += m_vUp.y * speed/2;
	//m_vPosition.z += m_vUp.z * speed/2;

	//m_vView.x	  += m_vUp.x * speed/2;
	m_vView.y	  += m_vUp.y * speed/2;
	//m_vView.z	  += m_vUp.z * speed/2;
}


void CCamera::straf(float speed)
{
	m_vPosition.x += m_vStrafe.x * speed*3;
	m_vPosition.z += m_vStrafe.z * speed*3;

	// Add the strafe vector to our view
	m_vView.x += m_vStrafe.x * speed;
	m_vView.z += m_vStrafe.z * speed;
}

void CCamera::rotate (float theta, float x, float y, float z)

{
	// temperary vectors
	CVector3 ViewVector;
	CVector3 NewVector;

	// vector from position of camera to view 
	ViewVector = m_vView - m_vPosition;
	

	//cos and sin angle
	float cosTheta = (float)cos(theta);
	float sinTheta = (float)sin(theta);

	//find the new x position
	NewVector.x  = (cosTheta + (1 - cosTheta) * x * x)		* ViewVector.x;
	NewVector.x += ((1 - cosTheta) * x * y - z * sinTheta)	* ViewVector.y;
	NewVector.x += ((1 - cosTheta) * x * z + y * sinTheta)	* ViewVector.z;

	//find the new y position
	NewVector.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* ViewVector.x;
	NewVector.y += (cosTheta + (1 - cosTheta) * y * y)		* ViewVector.y;
	NewVector.y += ((1 - cosTheta) * y * z - x * sinTheta)	* ViewVector.z;

	//find the new z position
	NewVector.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* ViewVector.x;
	NewVector.z += ((1 - cosTheta) * y * z + x * sinTheta)	* ViewVector.y;
	NewVector.z += (cosTheta + (1 - cosTheta) * z * z)		* ViewVector.z;

	//m_vView == the NewVectoradded to the old position
	m_vView = m_vPosition + NewVector;
}

void CCamera::mouseView (int x, int y)
{
	int middleX = SCREEN_WIDTH  >> 1; //bit shifting to divde by 2
	int middleY = SCREEN_HEIGHT >> 1; //quick way to halve numbers
	float angleY;	//rotation in Y
	float angleZ;	//rotation in Z


	//if mouse has not moved return
	if( (x == middleX) && (y == middleY) ) return;

	//moves are mouse pointer back to sceen mid point
	glutWarpPointer(middleX, middleY);	
	
	//angle = movement in X and Y. divided to reduce sensitivity (float is the value we want to return)
	angleY= (float) (middleX - x) / 1000.0f;
	angleZ= (float) (middleY - y) / 1000.0f;

	{
		CVector3 temp;
		
		CVector3 vAxis;// our soon to be axis to rotate the camera up and down
		vAxis = Cross(m_vView - m_vPosition, m_vUp);	//view vector and up vector 
		vAxis = Normalize(vAxis);//normalise this vector

		// Rotate around our perpendicular axis and along the y-axis
		rotate(angleZ, vAxis.x, vAxis.y, vAxis.z);	//rotate around our new axis, for the up and down
		rotate(angleY, 0, 1, 0); //rotate and our y axis for the left and right movement
	}
}

void CCamera::update()
	{
			
CVector3 AxisToMoveAlong = Cross(m_vView - m_vPosition , m_vUp);

m_vStrafe = Normalize(AxisToMoveAlong);

	
if (keyIndex[GLUT_KEY_LEFT])
	g_Camera.straf(-movement);

if (keyIndex[GLUT_KEY_RIGHT])
	g_Camera.straf(movement);

if (keyIndex[GLUT_KEY_UP])
	g_Camera.move(movement);
	
if (keyIndex[GLUT_KEY_DOWN])
	g_Camera.move(-movement);

if (keyIndex[GLUT_KEY_HOME])
	g_Camera.moveUp(movement*3);

if (keyIndex[GLUT_KEY_END])
	g_Camera.moveUp(-movement*3);

look();
glutPostRedisplay();
	}

void CCamera::setupDisplay()
{
	glEnable(GL_DEPTH_TEST);
	glClearColor (0.0, 0.0, 0.0, 0.0);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();	
	gluPerspective(45.0f,1, 0.5f, 150.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


